--------------------------------------------------------------------------------
-- CERN (BE-CO-HT)
-- Top level entity of CONV-TTL-BLO V1
-- http://www.ohwr.org/projects/conv-trig-blo
--------------------------------------------------------------------------------
--
-- unit name: conv_ttl_blo_v2.vhd
--
-- author: Theodor-Adrian Stana (t.stana@cern.ch)
--         Carlos Gil Soriano (gilsoriano@gmail.com)
--
-- version: 1.0
--
-- description: Top entity of CONV-TTL-BLO V1
--
-- dependencies:
--
--------------------------------------------------------------------------------
-- GNU LESSER GENERAL PUBLIC LICENSE
--------------------------------------------------------------------------------
-- This source file is free software; you can redistribute it and/or modify it
-- under the terms of the GNU Lesser General Public License as published by the
-- Free Software Foundation; either version 2.1 of the License, or (at your
-- option) any later version. This source is distributed in the hope that it
-- will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
-- of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
-- See the GNU Lesser General Public License for more details. You should have
-- received a copy of the GNU Lesser General Public License along with this
-- source; if not, download it from http://www.gnu.org/licenses/lgpl-2.1.html
--------------------------------------------------------------------------------
library IEEE;
library unisim;

use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;
use UNISIM.VCOMPONENTS.ALL;

use work.bicolor_led_ctrl_pkg.all;
use work.wishbone_pkg.all;

entity conv_ttl_blo_v2 is
  generic
  (
    g_nr_ttl_chan : natural := 6;
    g_nr_inv_chan : natural := 4
  );
  port 
  (
--    RST                  : in  std_logic;
--    CLK20_VCXO           : in  std_logic;
    FPGA_CLK_P           : in  std_logic; --Using the 125MHz clock
    FPGA_CLK_N           : in  std_logic;
    
    -- LEDs
    LED_CTRL0            : out std_logic;
    LED_CTRL0_OEN        : out std_logic;
    LED_CTRL1            : out std_logic;
    LED_CTRL1_OEN        : out std_logic;
    LED_MULTICAST_2_0    : out std_logic;
    LED_MULTICAST_3_1    : out std_logic;
    LED_WR_GMT_TTL_TTLN  : out std_logic;
    LED_WR_LINK_SYSERROR : out std_logic;
    LED_WR_OK_SYSPW      : out std_logic;
    LED_WR_OWNADDR_I2C   : out std_logic;
    
    -- I/Os for pulses
    PULSE_FRONT_LED_N    : out std_logic_vector(g_nr_ttl_chan downto 1);
    PULSE_REAR_LED_N     : out std_logic_vector(g_nr_ttl_chan downto 1);
--    FPGA_INPUT_TTL_N     : in  std_logic_vector(g_nr_ttl_chan downto 1);
--    FPGA_OUT_TTL         : out std_logic_vector(g_nr_ttl_chan downto 1);
--    FPGA_BLO_IN          : in  std_logic_vector(g_nr_ttl_chan downto 1);
--    FPGA_TRIG_BLO        : out std_logic_vector(g_nr_ttl_chan downto 1);
--    INV_IN_N             : in  std_logic_vector(g_nr_inv_chan downto 1);
--    INV_OUT              : out std_logic_vector(g_nr_inv_chan downto 1);
    
    -- Lines for the i2c_slave
    SCL_I                : in  std_logic;
    SCL_O                : out std_logic;
    SCL_OE               : out std_logic;
    SDA_I                : in  std_logic;
    SDA_O                : out std_logic;
    SDA_OE               : out std_logic;
    FPGA_GA              : in  std_logic_vector(4 downto 0);
    FPGA_GAP             : in  std_logic;
--    
--    -- Pins of the SPI interface to write into the Flash memory
--    FPGA_PROM_CCLK       : out std_logic;
--    FPGA_PROM_CSO_B_N    : out std_logic;
--    FPGA_PROM_DIN        : in  std_logic;
--    FPGA_PROM_MOSI       : out std_logic;
--      
    FPGA_OE              : out std_logic;
    FPGA_BLO_OE          : out std_logic;
    FPGA_TRIG_TTL_OE     : out std_logic;
    FPGA_INV_OE          : out std_logic;
    
    -- TTL/INV_TTL_N
    LEVEL                : in  std_logic;
    
--    EXTRA_SWITCH         : in  std_logic_vector(7 downto 1);
    
    -- It allows power sequencing of the
    -- 24V rail after a security given delay
    MR_N                 : out std_logic;

    -- RTM identifiers, should match with the expected values
    -- TODO: add matching
    FPGA_RTMM_N        : in std_logic_vector(2 downto 0);
    FPGA_RTMP_N        : in std_logic_vector(2 downto 0);

    -- DS18B20U+ thermometer bidirectional port
    THERMOMETER        : inout std_logic
  );
end conv_ttl_blo_v2;


architecture behav of conv_ttl_blo_v2 is

  --============================================================================
  -- Type declarations
  --============================================================================
  type t_pts_state is
    (
      ST_TEST00,
      ST_TEST11
    );
  --============================================================================
  -- Constant declarations
  --============================================================================
  -- Number of Wishbone masters and slaves, for wb_crossbar
  constant c_nr_masters        : NATURAL := 1;
  constant c_nr_slaves         : NATURAL := 2;

  -----------------------------------------
  -- Memory map
  -- * all registers are word-addressable
  -- * all registers are word aligned
  -----------------------------------------
  --    PTS         [0000-0004]
  --    ONEWIRE     [0008-000C]
  -----------------------------------------
  -- slave order definitions
  constant c_slv_pts_regs     : natural := 0;
  constant c_slv_onewire_mst  : natural := 1;

  -- base address definitions
  constant c_addr_pts_regs    : t_wishbone_address := x"00000000";
  constant c_addr_onewire_mst : t_wishbone_address := x"00000008";


  -- address mask definitions
  constant c_mask_pts_regs    : t_wishbone_address := x"FFFFFFF8"; 
  constant c_mask_onewire_mst : t_wishbone_address := x"FFFFFFF8";
  
  -- addresses constant for Wishbone crossbar
  constant c_addresses  : t_wishbone_address_array(c_nr_slaves-1 downto 0)
                                                          := (
                                                               0 => c_addr_pts_regs,
                                                               1 => c_addr_onewire_mst
                                                             );

  -- masks constant for Wishbone crossbar
  constant c_masks      : t_wishbone_address_array(c_nr_slaves-1 downto 0)
                                                          := (
                                                               0 => c_mask_pts_regs,
                                                               1 => c_mask_onewire_mst
                                                             );

  --============================================================================
  -- Component declarations
  --============================================================================
  -- Reset generator component
  -- (use: global reset generation, output reset generation)
  component reset_gen is
    generic
    (
      -- Reset time in number of clk_i cycles
      g_reset_time : positive := 5_000_000
    );
    port
    (
      clk_i   : in  std_logic;
      rst_n_o : out std_logic
    );
  end component reset_gen;

  -- RTM detector component
  -- (use: detect the presence of an RTM/P module)
  component rtm_detector is
    port 
    (
      rtmm_i    : in std_logic_vector(2 downto 0);
      rtmp_i    : in std_logic_vector(2 downto 0);
      rtmm_ok_o : out std_logic;
      rtmp_ok_o : out std_logic
    );
  end component rtm_detector;

  -- I2C bridge
  -- (use: convert I2C transfers into WB transfers on memmapped registers)  
  component vme64x_i2c is
    port
    (
      -- Clock, reset
      clk_i      : in  std_logic;
      rst_n_i    : in  std_logic;

      -- I2C lines
      sda_en_o   : out std_logic;
      sda_i      : in  std_logic;
      sda_o      : out std_logic;
      scl_en_o   : out std_logic;
      scl_i      : in  std_logic;
      scl_o      : out std_logic;

      -- I2C address and status
      i2c_addr_i : in  std_logic_vector(6 downto 0);
      i2c_done_o : out std_logic;

      -- Wishbone master signals
      wbm_stb_o  : out std_logic;
      wbm_cyc_o  : out std_logic;
      wbm_sel_o  : out std_logic_vector(3 downto 0);
      wbm_we_o   : out std_logic;
      wbm_dat_i  : in  std_logic_vector(31 downto 0);
      wbm_dat_o  : out std_logic_vector(31 downto 0);
      wbm_adr_o  : out std_logic_vector(31 downto 0);
      wbm_ack_i  : in  std_logic;
      wbm_rty_i  : in  std_logic;
      wbm_err_i  : in  std_logic
    );
  end component vme64x_i2c;
  
  -- Regs to test I2C operation
  component pts_regs is
    port (
      rst_n_i                                  : in     std_logic;
      clk_sys_i                                : in     std_logic;
      wb_adr_i                                 : in     std_logic_vector(0 downto 0);
      wb_dat_i                                 : in     std_logic_vector(31 downto 0);
      wb_dat_o                                 : out    std_logic_vector(31 downto 0);
      wb_cyc_i                                 : in     std_logic;
      wb_sel_i                                 : in     std_logic_vector(3 downto 0);
      wb_stb_i                                 : in     std_logic;
      wb_we_i                                  : in     std_logic;
      wb_ack_o                                 : out    std_logic;
      wb_stall_o                               : out    std_logic;
      -- Port for std_logic_vector field: 'current test' in reg: 'control register'
      pts_ctrl_crrt_test_o                     : out    std_logic_vector(3 downto 0);
      -- Port for std_logic_vector field: 'bits' in reg: 't00'
      pts_t00_bits_o                           : out    std_logic_vector(31 downto 0)
    );
  end component pts_regs;
  

  --============================================================================
  -- Signal declarations
  --============================================================================
  -- Clock signals
  signal clk_125                    : std_logic;

  -- Reset signals
  signal rst_n, rst                 : std_logic;

  -- RTM detection signals
  signal rtmm, rtmp                 : std_logic_vector(2 downto 0);
  signal rtmm_ok, rtmp_ok           : std_logic;

  -- Signals for pulse generation triggers
  signal trig                       : std_logic_vector(g_nr_ttl_chan downto 1);
  signal trig_inv                   : std_logic_vector(g_nr_inv_chan downto 1);
  signal trig_ttl, trig_blo         : std_logic_vector(g_nr_ttl_chan downto 1);

  -- Temporary signal for blocking and TTL pulse outputs
  signal pulse_outputs              : std_logic_vector(g_nr_ttl_chan downto 1);
  
  -- Temporary signal for inverted-TTL pulse outputs
  signal inv_outputs                : std_logic_vector(g_nr_inv_chan downto 1);
  
  -- Pulse status LED signals
  signal front_led_en               : std_logic_vector(g_nr_ttl_chan downto 1);
  signal rear_led_en                : std_logic_vector(g_nr_ttl_chan downto 1);
  signal pulse_leds                 : std_logic_vector(g_nr_ttl_chan downto 1);

  -- Output enable signals
  signal oe, ttl_oe, blo_oe, inv_oe : std_logic;

  -- Signal for controlling the bicolor LED matrix
  signal bicolor_led_state          : std_logic_vector(23 downto 0);

  -- Wishbone crossbar signals
  signal xbar_slave_in     : t_wishbone_slave_in_array   (c_nr_masters - 1 downto 0);
  signal xbar_slave_out    : t_wishbone_slave_out_array  (c_nr_masters - 1 downto 0);
  signal xbar_master_in    : t_wishbone_master_in_array  (c_nr_slaves  - 1 downto 0);
  signal xbar_master_out   : t_wishbone_master_out_array (c_nr_slaves  - 1 downto 0);
  
  -- I2C bridge signals
  signal i2c_done : std_logic;
  signal i2c_up   : std_logic;
  signal i2c_addr : std_logic_vector(6 downto 0);

  -- PTS-specific signals
  signal pts_state     : t_pts_state;

  signal pts_crrt_test : std_logic_vector(3 downto 0);
  signal cnt_halfsec   : unsigned(25 downto 0);
  signal led_seq       : unsigned( 4 downto 0);

  -- one-wire master signals
  signal owr_pwren     : std_logic_vector(0 downto 0);
  signal owr_en        : std_logic_vector(0 downto 0);
  signal owr_in        : std_logic_vector(0 downto 0);

begin

  --============================================================================
  -- Generate 125 MHz global signal from differential lines
  --============================================================================
  cmp_125_diff_buf: IBUFGDS
    generic map 
    (
      DIFF_TERM    => TRUE,
      IBUF_LOW_PWR => TRUE
    )
    port map 
    (
      I  => FPGA_CLK_P,
      IB => FPGA_CLK_N,
      O  => clk_125
    );

  --============================================================================
  -- Internal and external reset generation
  --============================================================================
  -- Configure reset generator for 96ms power-on reset
  cmp_reset_gen: reset_gen
    generic map
    (
      -- Reset time: 12 * 8ns * (10**6) = 96 ms
      g_reset_time => 12*(10**6)
    )
    port map
    (
      clk_i   => clk_125,
      rst_n_o => rst_n
    );
  
  rst  <= not rst_n;
  MR_N <= rst_n;
  
  --============================================================================
  -- Output enable logic
  --============================================================================
  -- The general output enable is set first and the blocking, TTL
  -- and INV output enable signals are set one clock cycle later.
  p_oe: process(clk_125)
  begin
    if rising_edge(clk_125) then
      if (rst_n = '0') then
        oe     <= '0';
        blo_oe <= '0';
        ttl_oe <= '0';
        inv_oe <= '0';
      else
        oe <= '0';
        if (oe = '1') then
          blo_oe <= '1';
          ttl_oe <= '1';
          inv_oe <= '1';
        end if;
      end if;
    end if;
  end process p_oe;

  FPGA_OE          <= oe;
  FPGA_BLO_OE      <= blo_oe;
  FPGA_TRIG_TTL_OE <= ttl_oe;
  FPGA_INV_OE      <= inv_oe;
  
  --============================================================================
  -- I2C bridge logic
  --============================================================================
  i2c_addr <= "10" & FPGA_GA;
  
  cmp_i2c_bridge: vme64x_i2c
    port map
    (
      -- Clock, reset
      clk_i           => clk_125,
      rst_n_i         => rst_n,

      -- I2C lines
      sda_en_o        => SDA_OE,
      sda_i           => SDA_I,
      sda_o           => SDA_O,
      scl_en_o        => SCL_OE,
      scl_i           => SCL_I,
      scl_o           => SCL_O,

      -- I2C address and status
      i2c_addr_i      => i2c_addr,
      i2c_done_o      => i2c_done,

      -- Wishbone master signals
      wbm_stb_o => xbar_slave_in(0).stb,
      wbm_cyc_o => xbar_slave_in(0).cyc,
      wbm_sel_o => xbar_slave_in(0).sel,
      wbm_we_o  => xbar_slave_in(0).we,
      wbm_dat_i => xbar_slave_out(0).dat,
      wbm_dat_o => xbar_slave_in(0).dat,
      wbm_adr_o => xbar_slave_in(0).adr,
      wbm_ack_i => xbar_slave_out(0).ack,
      wbm_rty_i => xbar_slave_out(0).rty,
      wbm_err_i => xbar_slave_out(0).err
    );
  
  -- Process to set the I2C_UP signal for display on the front panel
  -- of the front module. The I2C_UP signal is permanently set once an
  -- I2C transfer has successfully completed, as signaled by the RD_DONE
  -- and WR_DONE outputs of the I2C slave.
  p_i2c_up: process (clk_125) is
  begin
    if rising_edge(clk_125) then
      if (rst_n = '0') then
        i2c_up <= '0';
      elsif (i2c_done = '1') then
        i2c_up <= '1';
      end if;
    end if;
  end process p_i2c_up;

  --============================================================================
  -- PTS registers
  --============================================================================
  -- Regs to test I2C operation
  cmp_pts_regs: pts_regs
    port map
    (
      rst_n_i        => rst_n,
      clk_sys_i      => clk_125,
      wb_adr_i       => xbar_master_out(c_slv_pts_regs).adr(2 downto 2),
      wb_dat_i       => xbar_master_out(c_slv_pts_regs).dat,
      wb_dat_o       => xbar_master_in(c_slv_pts_regs).dat,
      wb_cyc_i       => xbar_master_out(c_slv_pts_regs).cyc,
      wb_sel_i       => xbar_master_out(c_slv_pts_regs).sel,
      wb_stb_i       => xbar_master_out(c_slv_pts_regs).stb,
      wb_we_i        => xbar_master_out(c_slv_pts_regs).we,
      wb_ack_o       => xbar_master_in(c_slv_pts_regs).ack,
      wb_stall_o     => xbar_master_in(c_slv_pts_regs).stall,

      -- PTS control register
      pts_ctrl_crrt_test_o => pts_crrt_test,

      -- PTS T00 register
      pts_t00_bits_o => open
    );

  --============================================================================
  -- Instantiation and connection of a Wishbone crossbar module
  --============================================================================
  -- xbar_master_in(0).stall <= '0';
  xbar_master_in(0).int <= '0';
  xbar_master_in(0).err <= '0';

  cmp_wb_crossbar: xwb_crossbar
  generic map
  (
    g_num_masters => c_nr_masters,
    g_num_slaves  => c_nr_slaves,
    g_registered  => false,
    g_address     => c_addresses,
    g_mask        => c_masks
  )
  port map
  (
    clk_sys_i => clk_125,
    rst_n_i   => rst_n,
    slave_i   => xbar_slave_in,
    slave_o   => xbar_slave_out,
    master_i  => xbar_master_in,
    master_o  => xbar_master_out
  );

  PULSE_FRONT_LED_N <= "111110" when (led_seq =  1) else
                       "111101" when (led_seq =  2) else 
                       "111011" when (led_seq =  3) else 
                       "110111" when (led_seq =  4) else 
                       "101111" when (led_seq =  5) else 
                       "011111" when (led_seq =  6) else 
                       "111110" when (led_seq =  7) else
                       "111101" when (led_seq =  8) else 
                       "111011" when (led_seq =  9) else 
                       "110111" when (led_seq = 10) else 
                       "101111" when (led_seq = 11) else 
                       "011111" when (led_seq = 12) else 
                       "111110" when (led_seq = 13) else
                       "111101" when (led_seq = 14) else 
                       "111011" when (led_seq = 15) else 
                       "110111" when (led_seq = 16) else 
                       "101111" when (led_seq = 17) else 
                       "011111" when (led_seq = 18) else 
                       "111110" when (led_seq = 19) else
                       "111101" when (led_seq = 20) else 
                       "111011" when (led_seq = 21) else 
                       "110111" when (led_seq = 22) else 
                       "101111" when (led_seq = 23) else 
                       "011111" when (led_seq = 24) else 
                       "111111";

  PULSE_REAR_LED_N <= "111110" when (led_seq =  1) else
                      "111101" when (led_seq =  2) else 
                      "111011" when (led_seq =  3) else 
                      "110111" when (led_seq =  4) else 
                      "101111" when (led_seq =  5) else 
                      "011111" when (led_seq =  6) else 
                      "111110" when (led_seq =  7) else
                      "111101" when (led_seq =  8) else 
                      "111011" when (led_seq =  9) else 
                      "110111" when (led_seq = 10) else 
                      "101111" when (led_seq = 11) else 
                      "011111" when (led_seq = 12) else 
                      "111110" when (led_seq = 13) else
                      "111101" when (led_seq = 14) else 
                      "111011" when (led_seq = 15) else 
                      "110111" when (led_seq = 16) else 
                      "101111" when (led_seq = 17) else 
                      "011111" when (led_seq = 18) else 
                      "111110" when (led_seq = 19) else
                      "111101" when (led_seq = 20) else 
                      "111011" when (led_seq = 21) else 
                      "110111" when (led_seq = 22) else 
                      "101111" when (led_seq = 23) else 
                      "011111" when (led_seq = 24) else 
                      "111111";

  --============================================================================
  -- FSM logic
  --============================================================================
  p_fsm: process(clk_125) is
  begin
    if rising_edge(clk_125) then
      if (rst_n = '0') then
        pts_state        <= ST_TEST00;
        cnt_halfsec      <= (others => '0');
        led_seq          <= (others => '0');
      else
        case pts_state is
          when ST_TEST00 =>
            if (pts_crrt_test = x"7") then
              pts_state <= ST_TEST11;
            end if;

          when ST_TEST11 =>
            cnt_halfsec <= cnt_halfsec + 1;
            if (cnt_halfsec = 62499999) then
              cnt_halfsec <= (others => '0');
              led_seq     <= led_seq + 1;
              if (led_seq = 24) then
                led_seq   <= (others => '0');
                pts_state <= ST_TEST00;
              end if;
            end if;

          when others =>
            pts_state <= ST_TEST00;

        end case;
      end if;
    end if;
  end process p_fsm;

  --============================================================================
  -- Bicolor LED matrix logic
  --============================================================================
  bicolor_led_state <= "000000000001000000000000" when (led_seq =  1) else
                       "000000000100000000000000" when (led_seq =  2) else
                       "000000010000000000000000" when (led_seq =  3) else
                       "000001000000000000000000" when (led_seq =  4) else
                       "000000000000000001000000" when (led_seq =  5) else
                       "000000000000000000010000" when (led_seq =  6) else
                       "000000000000000000000100" when (led_seq =  7) else
                       "000000000000000000000001" when (led_seq =  8) else
                       "000000000000000100000000" when (led_seq =  9) else
                       "000000000000010000000000" when (led_seq = 10) else
                       "000100000000000000000000" when (led_seq = 11) else
                       "010000000000000000000000" when (led_seq = 12) else
                       "000000000010000000000000" when (led_seq = 13) else
                       "000000001000000000000000" when (led_seq = 14) else
                       "000000100000000000000000" when (led_seq = 15) else
                       "000010000000000000000000" when (led_seq = 16) else
                       "000000000000000010000000" when (led_seq = 17) else
                       "000000000000000000100000" when (led_seq = 18) else
                       "000000000000000000001000" when (led_seq = 19) else
                       "000000000000000000000010" when (led_seq = 20) else
                       "000000000000001000000000" when (led_seq = 21) else
                       "000000000000100000000000" when (led_seq = 22) else
                       "001000000000000000000000" when (led_seq = 23) else
                       "100000000000000000000000" when (led_seq = 24) else
                       "000000000000000000000000";

  cmp_bicolor_led_ctrl: bicolor_led_ctrl
    generic map
    (
      g_NB_COLUMN    => 6,
      g_NB_LINE      => 2,
      g_clk_freq     => 125000000,
      g_refresh_rate => 250
    )
    port map
    (
      clk_i           => clk_125,
      rst_n_i         => rst_n,
      led_intensity_i => "0011111",
      led_state_i     => bicolor_led_state,
      column_o(0)     => LED_WR_OWNADDR_I2C,
      column_o(1)     => LED_WR_GMT_TTL_TTLN,
      column_o(2)     => LED_WR_LINK_SYSERROR,
      column_o(3)     => LED_WR_OK_SYSPW,
      column_o(4)     => LED_MULTICAST_2_0,
      column_o(5)     => LED_MULTICAST_3_1,
      line_o(0)       => LED_CTRL0,
      line_o(1)       => LED_CTRL1,
      line_oen_o(0)   => LED_CTRL0_OEN,
      line_oen_o(1)   => LED_CTRL1_OEN
    );

  --============================================================================
  -- One-wire master instantiation
  --============================================================================
  cmp_onewire_master: wb_onewire_master
    generic map
    (
      g_interface_mode      => CLASSIC,
      g_address_granularity => WORD,
      g_num_ports           => 1,
      g_ow_btp_normal       => "5.0",
      g_ow_btp_overdrive    => "1.0"
    )
    port map
    (
      clk_sys_i   => clk_125,
      rst_n_i     => rst_n,

      wb_cyc_i    => xbar_master_out(c_slv_onewire_mst).cyc,
      wb_sel_i    => xbar_master_out(c_slv_onewire_mst).sel,
      wb_stb_i    => xbar_master_out(c_slv_onewire_mst).stb,
      wb_we_i     => xbar_master_out(c_slv_onewire_mst).we,
      wb_adr_i    => xbar_master_out(c_slv_onewire_mst).adr(4 downto 2), 
      wb_dat_i    => xbar_master_out(c_slv_onewire_mst).dat,
      wb_dat_o    => xbar_master_in(c_slv_onewire_mst).dat,
      wb_ack_o    => xbar_master_in(c_slv_onewire_mst).ack, 
      wb_int_o    => open, 
      wb_stall_o  => xbar_master_in(c_slv_onewire_mst).stall,

      owr_pwren_o => owr_pwren,
      owr_en_o    => owr_en,
      owr_i       => owr_in
    );
     
  THERMOMETER <= '0' when (owr_en(0) = '1') else 
                 'Z';
  owr_in(0)   <= THERMOMETER;
  --============================================================================
  -- RTM detection logic
  --============================================================================
  rtmm <= not FPGA_RTMM_N;
  rtmp <= not FPGA_RTMP_N;

  cmp_rtm_detector: rtm_detector
    port map
    (
      rtmm_i    => rtmm,
      rtmp_i    => rtmp,
      rtmm_ok_o => rtmm_ok,
      rtmp_ok_o => rtmp_ok
    );

end behav;
